"""Test model factories provided by Improved User"""
from django.test import TestCase

from improved_user.factories import UserFactory
from improved_user.models import User


class UserFactoryTests(TestCase):
    """Test for UserFactory used with Factory Boy"""

    def test_basic_build(self):
        """Test creation of User via factory"""
        user = UserFactory.build()
        self.assertIsInstance(user, User)
        self.assertIsInstance(user.email, str)
        self.assertIsInstance(user.short_name, str)
        self.assertIsInstance(user.full_name, str)
        self.assertGreater(len(user.email), 1)
        self.assertGreater(len(user.short_name), 1)
        self.assertGreater(len(user.full_name), 1)
        self.assertTrue(user.check_password('password!'))
        self.assertTrue(user.is_active)
        self.assertFalse(user.is_staff)
        self.assertFalse(user.is_superuser)
        self.assertEqual(User.objects.all().count(), 0)
        user.save()
        self.assertEqual(User.objects.all().count(), 1)

    def test_basic_create(self):
        """Test creation of User via factory saves to DB"""
        user = UserFactory()
        self.assertIsInstance(user, User)
        self.assertEqual(User.objects.all().count(), 1)

    def test_attributes_override_build(self):
        """Test that all model fields can be modified"""
        user = UserFactory.build(
            email='hello@jambonsw.com',
            password='my_secret_password87',
            short_name='René',
            full_name='René Magritte',
            is_active=False,
            is_staff=True,
            is_superuser=True,
        )
        self.assertIsInstance(user, User)
        self.assertEqual(user.email, 'hello@jambonsw.com')
        self.assertEqual(user.short_name, 'René')
        self.assertEqual(user.full_name, 'René Magritte')
        self.assertTrue(user.check_password('my_secret_password87'))
        self.assertFalse(user.is_active)
        self.assertTrue(user.is_staff)
        self.assertTrue(user.is_superuser)
        self.assertEqual(User.objects.all().count(), 0)
        user.save()
        self.assertEqual(User.objects.all().count(), 1)

    def test_attributes_override_create(self):
        """Test that all model fields can be modified during creation"""
        user = UserFactory(
            email='hello@jambonsw.com',
            password='my_secret_password87',
            short_name='René',
            full_name='René Magritte',
            is_active=False,
            is_staff=True,
            is_superuser=True,
        )
        self.assertIsInstance(user, User)
        self.assertTrue(user.check_password('my_secret_password87'))
        self.assertEqual(User.objects.all().count(), 1)



"""Test User model management commands"""
from io import StringIO

from django.core.management import call_command
from django.core.management.base import CommandError
from django.test import TestCase

from improved_user.models import User


class CreatesuperuserManagementCommandTestCase(TestCase):
    """Test createsuperuser management command"""

    def test_basic_usage(self):
        """Check the operation of the createsuperuser management command"""
        # We can use the management command to create a superuser
        new_io = StringIO()
        call_command(
            'createsuperuser',
            interactive=False,
            email='joe@somewhere.org',
            short_name='Joe',
            full_name='Joe Smith',
            stdout=new_io,
        )
        command_output = new_io.getvalue().strip()
        self.assertEqual(command_output, 'Superuser created successfully.')
        user = User.objects.get(email='joe@somewhere.org')
        self.assertEqual(user.short_name, 'Joe')

        # created password should be unusable
        self.assertFalse(user.has_usable_password())

    def test_no_email_argument(self):
        """Ensure that email is required"""
        new_io = StringIO()
        with self.assertRaisesMessage(CommandError,
                                      'You must use --email with --noinput.'):
            call_command(
                'createsuperuser',
                interactive=False,
                short_name='Joe',
                full_name='Joe Smith',
                stdout=new_io)

    def test_invalid_username(self):
        """Creation fails if the username fails validation."""
        email_field = User._meta.get_field('email')
        new_io = StringIO()
        invalid_username = ('x' * email_field.max_length) + 'y'

        expected_error = (
            'Enter a valid email address.; '
            'Ensure this value has at most %d characters (it has %d).'
            % (email_field.max_length, len(invalid_username))
        )

        with self.assertRaisesMessage(CommandError, expected_error):
            call_command(
                'createsuperuser',
                interactive=False,
                email=invalid_username,
                short_name='Joe',
                full_name='Joe Smith',
                password='password1!',
                stdout=new_io)



"""Forms for Creating and Updating Improved Users"""
from django import VERSION as DjangoVersion, forms
from django.contrib.auth.forms import ReadOnlyPasswordHashField
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _

from .models import User

try:
    from django.contrib.auth import password_validation
except ImportError:  # pragma: no cover
    class EmptyValidator:
        """
        Class to mimic password validator API

        Django 1.8 doesn't have password strength validation
        We therefore introduce a mimic into the namespace

        """
        def validate_password(self, password, instance):
            """Accept password and user model and do nothing"""
            pass

        # pylint: disable=no-self-use
        def password_validators_help_text_html(self):
            """
            Used by password1 field;
            return None, as all strings are valid passwords

            """
            return None
        # pylint: enable=no-self-use

    password_validation = EmptyValidator()


class AbstractUserCreationForm(forms.ModelForm):
    """
    A form that creates a user, with no privileges, from the given
    username and password.
    """
    error_messages = {
        'password_mismatch': _("The two password fields didn't match."),
    }

    # TODO: move this to field when Django 1.8 support dropped
    password_kwargs = {'strip': False} if DjangoVersion >= (1, 9) else {}

    password1 = forms.CharField(
        label=_('Password'),
        widget=forms.PasswordInput,
        help_text=password_validation.password_validators_help_text_html(),
        **password_kwargs  # noqa: C815
    )
    password2 = forms.CharField(
        label=_('Password confirmation'),
        widget=forms.PasswordInput,
        help_text=_('Enter the same password as above, for verification.'),
        **password_kwargs  # noqa: C815
    )

    def clean_password2(self):
        """
        Check wether password 1 and password 2 are equivalent

        While ideally this would be done in clean, there is a chance a
        superclass could declare clean and forget to call super. We
        therefore opt to run this password mismatch check in password2
        clean, but to show the error above password1 (as we are unsure
        whether password 1 or password 2 contains the typo, and putting
        it above password 2 may lead some users to believe the typo is
        in just one).

        """
        password1 = self.cleaned_data.get('password1')
        password2 = self.cleaned_data.get('password2')
        if password1 and password2 and password1 != password2:
            self.add_error(
                'password1',
                forms.ValidationError(
                    self.error_messages['password_mismatch'],
                    code='password_mismatch',
                ))
        return password2

    def _post_clean(self):
        """Run password validaton after clean methods

        When clean methods are run, the user instance does not yet
        exist.  To properly compare model values agains the password (in
        the UserAttributeSimilarityValidator), we wait until we have an
        instance to compare against.

        https://code.djangoproject.com/ticket/28127
        https://github.com/django/django/pull/8408

        Has no effect in Django prior to 1.9
        May become unnecessary in Django 2.0 (if this superclass changes)

        """
        super()._post_clean()  # updates self.instance with form data
        password = self.cleaned_data.get('password1')
        if password:
            try:
                password_validation.validate_password(password, self.instance)
            except ValidationError as error:
                self.add_error('password1', error)

    def save(self, commit=True):
        """Save the user; use password hasher to set password"""
        user = super(AbstractUserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data['password1'])
        if commit:
            user.save()
        return user


class UserCreationForm(AbstractUserCreationForm):
    """
    A concrete implementation of AbstractUserCreationForm that uses an
    e-mail address as a user's identifier.
    """
    # TODO: when Py3.4 dropped, replace comprehension below with:
    # error_messages = {
    #     **AbstractUserCreationForm.error_messages,
    #     'duplicate_email': _('A user with that email already exists.'),
    # }
    error_messages = {
        k: v
        for d in [
            AbstractUserCreationForm.error_messages,
            {'duplicate_email': _('A user with that email already exists.')}]
        for k, v in d.items()
    }

    class Meta:
        model = User
        fields = ('email', 'full_name', 'short_name')

    def clean_email(self):
        """Clean email; set nice error message

        Since User.email is unique, this check is redundant,
        but it sets a nicer error message than the ORM. See #13147.

        https://code.djangoproject.com/ticket/13147
        """
        email = self.cleaned_data['email']
        try:
            # https://docs.djangoproject.com/en/stable/topics/db/managers/#default-managers
            # pylint: disable=protected-access
            User._default_manager.get(email=email)
            # pylint: enable=protected-access
        except User.DoesNotExist:
            return email
        raise forms.ValidationError(
            self.error_messages['duplicate_email'],
            code='duplicate_email',
        )


class AbstractUserChangeForm(forms.ModelForm):
    """Base form update User, but not their password"""
    password = ReadOnlyPasswordHashField(
        label=_('Password'),
        help_text=_(
            'Raw passwords are not stored, so there is no way to see this '
            "user's password, but you can change the password using "
            '<a href="{}">this form</a>.'),
    )

    rel_password_url = None

    def __init__(self, *args, **kwargs):
        """Initialize form; optimize user permission queryset"""
        super(AbstractUserChangeForm, self).__init__(*args, **kwargs)
        self.fields['password'].help_text = (
            self.fields['password'].help_text.format(
                self.get_local_password_path()))
        permission_field = self.fields.get('user_permissions', None)
        if permission_field is not None:
            # pre-load content types associated with permissions
            permission_field.queryset = (
                permission_field.queryset.select_related('content_type'))

    def get_local_password_path(self):
        """Method to return relative path to password form

        Will return rel_password_url attribute on form
        or else '../password/'. If subclasses cannot simply replace
        rel_password_url, then they can override this method instead of
        __init__.

        """
        if (hasattr(self, 'rel_password_url')
                and self.rel_password_url is not None):
            return self.rel_password_url
        if DjangoVersion < (1, 9):
            return './password/'
        return '../password/'

    def clean_password(self):
        """Change user info; not the password

        We seek to change the user, but not the password.
        Regardless of what the user provides, return the initial value.
        This is done here, rather than on the field, because the
        field does not have access to the initial value
        """
        return self.initial['password']


class UserChangeForm(AbstractUserChangeForm):
    """Form to update user, but not their password"""
    class Meta:
        model = User
        fields = '__all__'



"""Test User model manager"""
from datetime import datetime

from django.test import TestCase

from improved_user.models import User, UserManager


class UserManagerTestCase(TestCase):
    """Test User model manager"""

    def test_create_user_email_domain_normalize_rfc3696(self):
        """Normalize email allows @ in email local section"""
        # According to  http://tools.ietf.org/html/rfc3696#section-3
        # the "@" symbol can be part of the local part of an email address
        returned = UserManager.normalize_email(r'Abc\@DEF@EXAMPLE.com')
        self.assertEqual(returned, r'Abc\@DEF@example.com')

    def test_create_user_email_domain_normalize(self):
        """Normalize email lowercases domain"""
        returned = UserManager.normalize_email('normal@DOMAIN.COM')
        self.assertEqual(returned, 'normal@domain.com')

    def test_create_user_email_domain_normalize_with_whitespace(self):
        """Normalize email allows whitespace in email local section"""
        # pylint: disable=anomalous-backslash-in-string
        returned = UserManager.normalize_email('email\ with_whitespace@D.COM')
        self.assertEqual(returned, 'email\ with_whitespace@d.com')
        # pylint: enable=anomalous-backslash-in-string

    def test_empty_username(self):
        """Manager raises error if email is missing"""
        self.assertRaisesMessage(
            ValueError,
            'An email address must be provided.',
            User.objects.create_user,
            email='',
        )

    def test_create_user_is_staff(self):
        """Check is_staff attribute is respected"""
        email = 'normal@normal.com'
        user = User.objects.create_user(email, is_staff=True)
        self.assertEqual(user.email, email)
        self.assertTrue(user.is_staff)

    def test_create_user_is_active(self):
        """Check is_active attribute is respected"""
        email = 'normal@normal.com'
        user = User.objects.create_user(email, is_active=False)
        self.assertEqual(user.email, email)
        self.assertFalse(user.is_active)

    def test_username_keyword_raises_warning(self):
        """Remind dev that username doesn't exist on model"""
        error = ('The Improved User model does not have a username; '
                 'it uses only email')
        with self.assertRaisesMessage(ValueError, error):
            User.objects.create_user(
                username='whoops',
                email='test@test.com',
                password='test',
            )
        with self.assertRaisesMessage(ValueError, error):
            User.objects.create_superuser(
                username='whoops',
                email='test@test.com',
                password='test',
            )

    def test_create_super_user_raises_error_on_false_is_superuser(self):
        """Warn developer when creating superuse without is_superuser"""
        error = 'Superuser must have is_superuser=True.'
        with self.assertRaisesMessage(ValueError, error):
            User.objects.create_superuser(
                email='test@test.com',
                password='test',
                is_superuser=False,
            )

    def test_create_superuser_raises_error_on_false_is_staff(self):
        """Warn developer when creating superuse without is_staff"""
        error = 'Superuser must have is_staff=True.'
        with self.assertRaisesMessage(ValueError, error):
            User.objects.create_superuser(
                email='test@test.com',
                password='test',
                is_staff=False,
            )

    def test_make_random_password(self):
        """Test manager make_random_password method"""
        allowed_chars = 'abcdefg'
        password = UserManager().make_random_password(5, allowed_chars)
        self.assertEqual(len(password), 5)
        for char in password:
            self.assertIn(char, allowed_chars)

    def test_last_login_is_none(self):
        """Check that last login is unset when created

        https://github.com/jambonsw/django-improved-user/issues/25
        """
        user1 = User.objects.create_user('hello@jambonsw.com', 'password1')
        self.assertIsNone(user1.last_login)

        user2 = User.objects.create_superuser('clark@kent.com', 'password1')
        self.assertIsNone(user2.last_login)

    def test_date_joined_default(self):
        """Check date joined set upon creation"""
        user1 = User.objects.create_user('hello@jambonsw.com', 'password1')
        self.assertIsNotNone(user1.date_joined)
        self.assertIsInstance(user1.date_joined, datetime)

        user2 = User.objects.create_superuser('clark@kent.com', 'password1')
        self.assertIsNotNone(user2.date_joined)
        self.assertIsInstance(user2.date_joined, datetime)



"""Test UserCreationForm and UserChangeForm"""
from unittest import skipUnless
from unittest.mock import patch

from django import VERSION as DjangoVersion
from django.forms.fields import Field
from django.test import TestCase, override_settings
from django.utils.translation import gettext as _

from improved_user.forms import UserChangeForm, UserCreationForm
from improved_user.models import User


class TestDataMixin:  # pylint: disable=too-few-public-methods
    """Mixin used by both test classes below; creates users"""

    @classmethod
    def setUpTestData(cls):
        """Called by TestCase during initialization; creates users"""
        cls.u1 = User.objects.create_user(
            email='testclient@example.com',
            password='password',
            short_name='tester')
        cls.u2 = User.objects.create_user(
            email='inactive@example.com',
            password='password',
            is_active=False)
        cls.u3 = User.objects.create_user(
            email='staff@example.com',
            password='password')
        cls.u4 = User.objects.create(
            email='empty_password@example.com',
            password='')
        cls.u5 = User.objects.create(
            email='unmanageable_password@example.com',
            password='$')
        cls.u6 = User.objects.create(
            email='unknown_password@example.com',
            password='foo$bar')


class UserCreationFormTest(TestDataMixin, TestCase):
    """Test UserCreationForm with Improved User"""

    def test_user_already_exists(self):
        """Raise errors if user already exists"""
        data = {
            'email': 'testclient@example.com',
            'password1': 'test123',
            'password2': 'test123',
        }
        form = UserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertEqual(form['email'].errors,
                         [str(form.error_messages['duplicate_email'])])

    def test_invalid_data(self):
        """Raise errors if invalid email format"""
        data = {
            'email': '%%%',
            'password1': 'test123',
            'password2': 'test123',
        }
        form = UserCreationForm(data)
        self.assertFalse(form.is_valid())
        validator = next(v
                         for v in User._meta.get_field('email').validators
                         if v.code == 'invalid')
        self.assertEqual(form['email'].errors, [str(validator.message)])

    def test_password_verification(self):
        """The verification password is incorrect."""
        data = {
            'email': 'jsmith@example.com',
            'password1': 'test123',
            'password2': 'test',
        }
        form = UserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertEqual(form['password1'].errors,
                         [str(form.error_messages['password_mismatch'])])

    def test_both_passwords(self):
        """One (or both) passwords weren't given"""
        data = {'email': 'jsmith@example.com'}
        form = UserCreationForm(data)
        required_error = [str(Field.default_error_messages['required'])]
        self.assertFalse(form.is_valid())
        self.assertEqual(form['password1'].errors, required_error)
        self.assertEqual(form['password2'].errors, required_error)

        data['password2'] = 'test123'
        form = UserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertEqual(form['password1'].errors, required_error)
        self.assertEqual(form['password2'].errors, [])

    @skipUnless(
        DjangoVersion >= (1, 9),
        'Password strength checks not available on Django 1.8')
    @patch('django.contrib.auth.password_validation.password_changed')
    def test_success(self, password_changed):
        """Successful submission of form data"""
        data = {
            'email': 'jsmith@example.com',
            'full_name': 'John Smith',  # optional field
            'short_name': 'John',  # optional field
            'password1': 'test123',
            'password2': 'test123',
        }
        form = UserCreationForm(data)
        self.assertTrue(form.is_valid())
        form.save(commit=False)
        self.assertEqual(password_changed.call_count, 0)
        user = form.save()
        self.assertEqual(password_changed.call_count, 1)
        self.assertEqual(repr(user), '<User: jsmith@example.com>')
        self.assertEqual(user.get_short_name(), 'John')
        self.assertEqual(user.get_full_name(), 'John Smith')
        self.assertTrue(user.check_password('test123'))

    # TODO: Remove this test in favor of above after Dj1.8 dropped
    @skipUnless(
        DjangoVersion < (1, 9),
        'Password strength checks not available on Django 1.8')
    def test_success_pre_19(self):
        """Successful submission of form data"""
        data = {
            'email': 'jsmith@example.com',
            'full_name': 'John Smith',  # optional field
            'short_name': 'John',  # optional field
            'password1': 'test123',
            'password2': 'test123',
        }
        form = UserCreationForm(data)
        self.assertTrue(form.is_valid())
        user = form.save()
        self.assertEqual(repr(user), '<User: jsmith@example.com>')
        self.assertEqual(user.get_short_name(), 'John')
        self.assertEqual(user.get_full_name(), 'John Smith')
        self.assertTrue(user.check_password('test123'))

    @skipUnless(
        DjangoVersion >= (1, 9),
        'Password strength checks not available on Django 1.8')
    @override_settings(
        AUTH_PASSWORD_VALIDATORS=[
            {'NAME': 'django.contrib.auth.password_validation.'
                     'CommonPasswordValidator'},
            {'NAME': 'django.contrib.auth.password_validation.'
                     'MinimumLengthValidator',
             'OPTIONS': {'min_length': 12}}])
    def test_common_password(self):
        """Ensure form works with Password Validation"""
        data = {
            'email': 'jsmith@example.com',
            'password1': 'password',
            'password2': 'password',
        }
        form = UserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertEqual(len(form['password1'].errors), 2)
        self.assertIn(
            'This password is too common.',
            form['password1'].errors)
        self.assertIn(
            'This password is too short. '
            'It must contain at least 12 characters.',
            form['password1'].errors)

    @skipUnless(
        DjangoVersion >= (1, 9),
        'Password strength checks not available on Django 1.8')
    @override_settings(
        AUTH_PASSWORD_VALIDATORS=[
            {'NAME': 'django.contrib.auth.password_validation.'
                     'UserAttributeSimilarityValidator'}])
    def test_validates_password_similarity_length(self):
        """Test misconfigured-similarity validator catches email"""
        data = {
            'email': 'jsmith@example.com',
            'password1': 'jsmith',
            'password2': 'jsmith',
        }
        form = UserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertIn(
            'The password is too similar to the email address.',
            form['password1'].errors)

    @skipUnless(
        DjangoVersion >= (1, 9),
        'Password strength checks not available on Django 1.8')
    @override_settings(
        AUTH_PASSWORD_VALIDATORS=[{
            'NAME': 'django.contrib.auth.password_validation.'
                    'UserAttributeSimilarityValidator',
            'OPTIONS': {
                'user_attributes': ('email', 'full_name', 'short_name')}}])
    def test_password_help_text(self):
        """Ensure UserAttributeSimilarityValidator help text is shown"""
        form = UserCreationForm()
        self.assertEqual(
            form.fields['password1'].help_text,
            '<ul><li>Your password can&#39;t be too similar to'
            ' your other personal information.</li></ul>')

    @skipUnless(
        DjangoVersion >= (1, 9),
        'Password strength checks not available on Django 1.8')
    @override_settings(
        AUTH_PASSWORD_VALIDATORS=[{
            'NAME': 'django.contrib.auth.password_validation.'
                    'UserAttributeSimilarityValidator',
            'OPTIONS': {
                'user_attributes': ('email', 'full_name', 'short_name')}}])
    def test_validates_password_with_all_data(self):
        """Test correctly configured similarity validatior catches name"""
        data = {
            'email': 'jsmith@example.com',
            'password1': 'johndoesmith',
            'password2': 'johndoesmith',
            'short_name': 'John',
            'full_name': 'John Smith',
        }
        form = UserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertEqual(
            form['password1'].errors,
            ['The password is too similar to the full name.'])

    def test_password_whitespace_not_stripped(self):
        """Ensure we aren't mangling passwords by removing whitespaces

        Starting in Django 1.9, form Charfield and subclasses allow for
        whitestrip to be stripped automatically during the clean field
        phase. This is a test to ensure that we are not stripping
        whitespace from the password.

        """
        data = {
            'email': 'jsmith@example.com',
            'password1': '   test password   ',
            'password2': '   test password   ',
        }
        form = UserCreationForm(data)
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['password1'], data['password1'])
        self.assertEqual(form.cleaned_data['password2'], data['password2'])


class UserChangeFormTest(TestDataMixin, TestCase):
    """Tests for UserChangeForm"""

    def test_email_validity(self):
        """Ensure Email Validation is used on User"""
        user = User.objects.get(email='testclient@example.com')
        data = {'email': 'not valid'}
        form = UserChangeForm(data, instance=user)
        self.assertFalse(form.is_valid())
        validator = next(v
                         for v in User._meta.get_field('email').validators
                         if v.code == 'invalid')
        self.assertEqual(form['email'].errors, [str(validator.message)])

    def test_bug_14242(self):
        """Regression test: allow UserChangeForm subclasses

        https://code.djangoproject.com/ticket/14242

        """
        class MyUserForm(UserChangeForm):
            """Custom Subclass to check lack of user_permissions field"""
            def __init__(self, *args, **kwargs):
                super(MyUserForm, self).__init__(*args, **kwargs)
                self.fields['groups'].help_text = (
                    'These groups give users different permissions')

            class Meta(UserChangeForm.Meta):
                fields = ('groups',)

        # Just check we can create it
        MyUserForm({})

    def test_unusable_password(self):
        """Test that Django shows unusable password warning"""
        user = User.objects.get(email='empty_password@example.com')
        user.set_unusable_password()
        user.save()
        form = UserChangeForm(instance=user)
        self.assertIn(_('No password set.'), form.as_table())

    def test_bug_17944_empty_password(self):
        """Test form can be used when password not set

        https://code.djangoproject.com/ticket/17944

        """
        user = User.objects.get(email='empty_password@example.com')
        form = UserChangeForm(instance=user)
        self.assertIn(_('No password set.'), form.as_table())

    def test_bug_17944_unmanageable_password(self):
        """Test form can be used when password unmanageable

        https://code.djangoproject.com/ticket/17944

        """
        user = User.objects.get(email='unmanageable_password@example.com')
        form = UserChangeForm(instance=user)
        self.assertIn(
            _('Invalid password format or unknown hashing algorithm.'),
            form.as_table())

    def test_bug_17944_unknown_password_algorithm(self):
        """Test form can be used when hashing algorithm is unrecognized

        https://code.djangoproject.com/ticket/17944

        """
        user = User.objects.get(email='unknown_password@example.com')
        form = UserChangeForm(instance=user)
        self.assertIn(
            _('Invalid password format or unknown hashing algorithm.'),
            form.as_table())

    def test_bug_19133(self):
        """The change form does not return the password value

        https://code.djangoproject.com/ticket/19133

        """
        # Use the form to construct the POST data
        user = User.objects.get(email='testclient@example.com')
        form_for_data = UserChangeForm(instance=user)
        post_data = form_for_data.initial

        # The password field should be readonly, so anything
        # posted here should be ignored; the form will be
        # valid, and give back the 'initial' value for the
        # password field.
        post_data['password'] = 'new password'
        form = UserChangeForm(instance=user, data=post_data)

        self.assertTrue(form.is_valid())
        self.assertTrue(user.check_password('password'))  # $ not in password
        self.assertIn('$', form.cleaned_data['password'])  # hash contains $

    def test_bug_19349_bound_password_field(self):
        """Test re-rendering of ReadOnlyPasswordHashWidget

        https://code.djangoproject.com/ticket/19349

        """
        user = User.objects.get(email='testclient@example.com')
        form = UserChangeForm(data={}, instance=user)
        # When rendering the bound password field,
        # ReadOnlyPasswordHashWidget needs the initial
        # value to render correctly
        self.assertEqual(form.initial['password'], form['password'].value())



"""App Configuration for Improved User"""
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class ImprovedUserConfig(AppConfig):
    """App Config for Improved User"""
    name = 'improved_user'
    verbose_name = _('Improved User')



"""Test URLs for auth admins"""
from django.conf.urls import url
from django.contrib import admin
from django.contrib.auth.admin import GroupAdmin
from django.contrib.auth.models import Group
from django.contrib.auth.urls import urlpatterns

from improved_user.admin import UserAdmin
from improved_user.models import User

# Create a silo'd admin site for just the user/group admins.
SITE = admin.AdminSite(name='auth_test_admin')
SITE.register(User, UserAdmin)
SITE.register(Group, GroupAdmin)

urlpatterns += [
    url(r'^admin/', SITE.urls),
]
